@@ -24,7 +24,7 @@ try:
   from io import StringIO
 except:
   from StringIO import StringIO
-from itertools import chain
+from itertools import chain, groupby, islice, product
 from lxml import etree
 
 from lnkckr.checkers.base import Checker as BaseChecker
@@ -78,20 +78,79 @@ class Checker(BaseChecker):
           links[link]['posts'].append(post_link)
     self.links = links
 
-  # =====
+  ###########################
+  # report output functions #
+  ###########################
+
+  def print_all(self):
+
+    self.print_report()
+    self.print_summary()
+    self.print_toplist()
+
+  # report
+  #########
 
   def print_report_link_data(self, url, link):
 
     for post in link['posts']:
       print('  %s' % post)
-    print()
 
-  # -----
+  # summary
+  ##########
 
-  def print_summary_status(self, status, links):
+  def print_summary_status(self, data):
+
+    data2 = []
+    for status, links in data:
+      nlinks = sum(1 for l in links)
+      nposts = len(set(chain.from_iterable(link['posts'] for link in links)))
+      data2.append((status, nlinks, nposts))
+
+    l_links = max(self.num_len(item[1]) for item in data2)
+    l_posts = max(self.num_len(item[2]) for item in data2)
+
+    for status, nlinks, nposts in data2:
+      cstatus = self.color_status(status)
+
+      print('{} {:{l_links},d} links from {:{l_posts},d} posts'.format(cstatus,
+            nlinks, nposts, l_links=l_links, l_posts=l_posts))
+
+  def print_summary_status1(self, status, links):
 
     nlinks = len(links)
     nposts = len(set(chain.from_iterable(link['posts'] for link in links)))
     cstatus = self.color_status(status)
 
-    print('%s %5d links from %5d posts' % (cstatus, nlinks, nposts))
+    print('{} {:6,d} links from {:6,d} posts'.format(cstatus, nlinks, nposts))
+
+  def print_summary_footer(self):
+
+    links = self.links
+    posts = set(chain.from_iterable(link['posts'] for link in links.values()))
+    nposts = len(posts)
+
+    print()
+    print('TOTAL {:,} links from {:,} posts'.format(len(links), nposts))
+    print()
+
+  # toplist
+  ##########
+
+  def print_toplist(self):
+
+    self.print_heading('toplist')
+
+    # make list of (status, postlink) from links
+    f = lambda link: link['status'] not in (None, '200', 'SCH', 'SKP')
+    links = filter(f, self.links.values())
+    links = (product((link['status'],), link['posts']) for link in links)
+    links = chain.from_iterable(links)
+    f = lambda link: link[1]
+    links = groupby(sorted(links, key=f), key=f)
+    links = ((post, sum(1 for _ in g)) for post, g in links)
+
+    print('%6s %s' % ('Errors', 'Post URL'))
+    for post, count in islice(sorted(links, key=f, reverse=True), 10):
+      print('%6d %s' % (count, post))
+    print()
